@@ -171,6 +171,10 @@ PGrnReleaseSequentialSearch(ResourceReleasePhase phase,
 		}
 		else
 		{
+			if (datum == currentDatum)
+			{
+				currentDatum = NULL;
+			}
 			PGrnSequentialSearchDatumFinalize(datum);
 			grn_hash_cursor_delete(ctx, cursor, NULL);
 		}
@@ -250,7 +254,7 @@ PGrnSequentialSearchPrepareIndex(PGrnCondition *condition,
 	grn_column_flags indexFlags = GRN_OBJ_COLUMN_INDEX;
 	bool targetIsVector =
 		(currentTargetType == PGRN_SEQUENTIAL_SEARCH_TARGET_TEXTS);
-	PGrnSequentialSearchDatumKey key;
+	PGrnSequentialSearchDatumKey key = {0};
 
 	if (!PGrnPGTextIsEmpty(condition->schemaName))
 	{
@@ -475,6 +479,7 @@ PGrnSequentialSearchPrepareExpression(PGrnCondition *condition,
 	{
 		grn_obj_close(ctx, currentDatum->expression);
 		currentDatum->expression = NULL;
+		currentDatum->expressionHash = 0;
 	}
 
 	GRN_EXPR_CREATE_FOR_QUERY(ctx,
